unit DString;

interface

uses
   WinTypes, SysUtils, Messages, Classes, Forms, DsgnIntF, Controls, Grids;

type
  EDGridError = class(Exception);
  TAboutDGrid = class(TPropertyEditor)
  public
      function GetAttributes: TPropertyAttributes; override;
      procedure Edit; override;
      function GetValue: string; override;
  end;
  TDStringGrid = class(TStringGrid)
  public
    procedure RemoveRows(RowIndex, RCount: LongInt);
    procedure InsertRows(RowIndex, RCount: LongInt);
    procedure RemoveCols(ColIndex, CCount: LongInt);
    procedure InsertCols(ColIndex, CCount: LongInt);
    procedure Clear;
    function isCell(SubStr: String; var ACol, ARow: LongInt): Boolean;
    procedure SaveToFile(FileName: String);
    procedure LoadFromFile(FileName: String);
    function CellToReal(ACol, ARow: LongInt): Real;
  private
    FAbout : TAboutDGrid;
  published
    property About: TAboutDGrid
             read FAbout write FAbout;
  end;

procedure Register;

implementation

procedure TDStringGrid.RemoveRows(RowIndex, RCount : LongInt);
var
  i: LongInt;
begin
  for i := RowIndex to RowCount - 1 do Rows[i] := Rows[i + RCount];
  RowCount := RowCount - RCount;
end;

procedure TAboutDGrid.Edit;
begin
 Application.MessageBox('by David Martinz: david@baydenet.com.br '+#10#13+
   'This component is FreeWare',' TDStringGrid version 1.0',MB_ICONASTERISK);
end;

function TAboutDGrid.GetValue: string;
begin
 Result := 'About TDStringGrid';
end;

function TAboutDGrid.GetAttributes: TPropertyAttributes;
begin
 Result := [paMultiSelect, paDialog, paReadOnly];
end;

procedure TDStringGrid.InsertRows(RowIndex, RCount : LongInt);
var
  i: LongInt;
begin
  RowCount := RowCount + RCount;
  for i := RowCount - 1 downto RowIndex do Rows[i] := Rows[i - RCount];
end;

procedure TDStringGrid.RemoveCols(ColIndex, CCount : LongInt);
var
  i: LongInt;
begin
  for i := ColIndex to ColCount - 1 do Cols[i] := Cols[i + CCount];
  ColCount := ColCount - CCount;
end;

procedure TDStringGrid.InsertCols(ColIndex, CCount : LongInt);
var
  i: LongInt;
begin
  ColCount := ColCount + CCount;
  for i := ColCount - 1 downto ColIndex do Cols[i] := Cols[i - CCount];
end;

procedure TDStringGrid.Clear;
var
  i: LongInt;
begin
  for i:= 0 to ColCount - 1 do Cols[i].Clear;
end;

function TDStringGrid.isCell(SubStr: String; var ACol, ARow: LongInt): Boolean;
var
  i, j: LongInt;
begin
  for i := 0 to RowCount - 1 do
  begin
    for j := 0 to ColCount - 1 do
    begin
      if Rows[i].Strings[j] = SubStr then
      begin
        ARow := i;
        ACol := j;
        Result := True;
        exit;
      end;
    end;
  end;
  Result := False;
end;

procedure TDStringGrid.SaveToFile(FileName: String);
var
  I, J: Integer;
  S: string;
  f: TextFile;
begin
  AssignFile(f, FileName);
  Rewrite(f);
  for i := 0 to RowCount - 1 do
  begin
    for j := 0 to ColCount - 1 do
    begin
      S := '#' + IntToStr(i) + ',' + IntToStr(j) + ' ' + Cells[j, i];
      if (Length(S) > Length(IntToStr(i)) + Length(IntToStr(j)) + 3) then
      Writeln(f, S);
    end;
  end;
  CloseFile(f);
end;

procedure TDStringGrid.LoadFromFile(FileName: String);
var
  X, Y, L: Integer;
  S, SI: string;
  F: TextFile;
begin
  AssignFile(f, FileName);
  Reset(f);
  if IOResult <> 0 then raise EDGridError.Create('File ' + FileName + ' not found.');
  X:=0;
  Y:=0;
  while not Eof(f) do
  begin
    Readln(f, S);
    S:=S + ' ';
    while not (S = '') do
      begin
        SI := Copy(S, 1, Pos(' ', S) - 1);
        S := Copy(S, Pos(' ', S) + 1, Length(S));
        if Copy(SI, 1, 1) = '#' then
          begin
            L:=Length(SI);
            X:=StrToInt(Copy(SI, Pos('#', SI)+1, Pos(',',SI)-2));
            Y:=StrToInt(Copy(SI, Pos(',', SI)+1, L - Pos(',', SI)));
          end;
        if (SI <> ' ') and (SI <> '') and (Copy(SI, 1, 1) <> '#') then
        begin
          Cells[Y,X]:=SI;
          Inc(Y);
        end;
      end;
    Y:=0;
    Inc(X);
  end;
  CloseFile(f);
end;

function TDStringGrid.CellToReal(ACol, ARow: LongInt): Real;
var
  i: Real;
  Code: Integer;
begin
  if Cells[ACol, ARow] <> '' then
  begin
    Val(Cells[ACol, ARow], i, Code);
    if Code <> 0 then raise
          EDGridError.Create('Erro na posio: ' +
          IntToStr(Code) + ' na Clula [' + IntToStr(ACol) + ', ' +
          IntToStr(ARow) + '].')
    else
    Result := i;
  end;
end;

procedure Register;
begin
  RegisterComponents('Sample32', [TDStringGrid]);
  RegisterPropertyEditor(TypeInfo(TAboutDGrid), TDStringGrid, 'ABOUT', TAboutDGrid);
end;

end.
